import logging

from webexteamssdk.models.cards import Colors, TextBlock, FontWeight, FontSize, Column, AdaptiveCard, ColumnSet, \
    ImageSize, Image, Fact
from webexteamssdk.models.cards.actions import Submit

from webex_bot_ecoen66.models.command import Command, COMMAND_KEYWORD_KEY
from webex_bot_ecoen66.models.response import response_from_adaptive_card

log = logging.getLogger(__name__)

HELP_COMMAND_KEYWORD = "help"


class HelpCommand(Command):

    def __init__(self, bot_name, bot_help_subtitle, bot_help_image, bot_help_image_size = ImageSize.SMALL):
        self.commands = None
        super().__init__(
            command_keyword=HELP_COMMAND_KEYWORD,
            help_message="Get Help",
            card=None)
        self.card_callback = self.build_card
        self.card_populated = False
        self.bot_name = bot_name
        self.bot_help_subtitle = bot_help_subtitle
        self.bot_help_image = bot_help_image
        self.bot_help_image_size = bot_help_image_size

    def execute(self, message, attachment_actions, activity):
        pass

    def build_card(self, message, attachment_actions, activity):
        """
        Construct a help message for users.
        :param message: message with command already stripped
        :param attachment_actions: attachment_actions object
        :param activity: activity object
        :return:
        """
        heading = TextBlock(self.bot_name, weight=FontWeight.BOLDER, wrap=True, size=FontSize.LARGE)
        subtitle = TextBlock(self.bot_help_subtitle, wrap=True, size=FontSize.SMALL, color=Colors.LIGHT)

        image = Image(
            url=self.bot_help_image,
            size=self.bot_help_image_size)

        header_column = Column(items=[heading, subtitle], width=2)
        header_image_column = Column(
            items=[image],
            width=1,
        )

        log.info(f"build help card activity: {activity}")
        thread_parent_id = None
        if 'parent' not in activity:
            thread_parent_id = activity['id']

        actions, hint_texts = self.build_actions_and_hints(thread_parent_id)

        card = AdaptiveCard(
            body=[ColumnSet(columns=[header_column, header_image_column]),
                  # ColumnSet(columns=[Column(items=[subtitle])]),
                  # FactSet(facts=hint_texts),
                  ],
            actions=actions)

        return response_from_adaptive_card(adaptive_card=card)

    def build_actions_and_hints(self, thread_parent_id):
        # help_card = HELP_CARD_CONTENT
        help_actions = []
        hint_texts = []

        if self.commands is not None:
            # Sort list by keyword
            sorted_commands_list = sorted(self.commands, key=lambda command: (
                command.command_keyword is not None, command.command_keyword))
            for command in sorted_commands_list:
                if command.help_message and command.command_keyword != HELP_COMMAND_KEYWORD:
                    action = Submit(
                        title=f"{command.help_message}",
                        data={COMMAND_KEYWORD_KEY: command.command_keyword,
                              'thread_parent_id': thread_parent_id},
                    )
                    help_actions.append(action)

                    hint = Fact(title=command.command_keyword,
                                value=command.help_message)

                    hint_texts.append(hint)
        return help_actions, hint_texts



"""Main module."""
import logging
import os

import backoff
import coloredlogs
import requests
import webexteamssdk

from webex_bot_ecoen66.commands.echo import EchoCommand
from webex_bot_ecoen66.commands.help import HelpCommand
from webex_bot_ecoen66.exceptions import BotException
from webex_bot_ecoen66.formatting import quote_info
from webex_bot_ecoen66.models.command import CALLBACK_KEYWORD_KEY, Command, COMMAND_KEYWORD_KEY
from webex_bot_ecoen66.models.response import Response
from webex_bot_ecoen66.websockets.webex_websocket_client import WebexWebsocketClient, DEFAULT_DEVICE_URL

log = logging.getLogger(__name__)


class WebexBot(WebexWebsocketClient):

    def __init__(self,
                 teams_bot_token,
                 approved_users=[],
                 approved_domains=[],
                 approved_rooms=[],
                 device_url=DEFAULT_DEVICE_URL,
                 include_demo_commands=False,
                 bot_name="Webex Bot",
                 bot_help_subtitle="Here are my available commands. Click one to begin.",
                 threads=True,
                 help_command=None,
                 log_level="INFO"):
        """
        Initialise WebexBot.

        @param teams_bot_token: Your token.
        @param approved_users: List of email address who are allowed to chat to this bot.
        @param approved_domains: List of domains which are allowed to chat to this bot.
        @param approved_rooms: List of rooms whose members are allowed to chat to this bot.
        @param device_url: WDM Url
        @param include_demo_commands: If True, any demo commands will be included.
        @param bot_name: Your custom name for the bot.
        @param bot_help_subtitle: Text to show in the help card.
        @param threads: If True, respond to msg by creating a thread.
        @param help_command: If None, use internal HelpCommand, otherwise override.
        @param log_level: Set loggin level.

        """

        coloredlogs.install(level=os.getenv("LOG_LEVEL", log_level),
                            fmt='%(asctime)s  [%(levelname)s]  '
                                '[%(module)s.%(name)s.%(funcName)'
                                's]:%(lineno)s %(message)s')
        log.info("Registering bot with Webex cloud")
        WebexWebsocketClient.__init__(self,
                                      teams_bot_token,
                                      on_message=self.process_incoming_message,
                                      on_card_action=self.process_incoming_card_action,
                                      device_url=device_url)

        # A dictionary of commands this bot listens to
        # Each key in the dictionary is a command, with associated help
        # text and callback function
        # By default supports 2 command, echo and help

        if help_command is None:
            self.help_command = HelpCommand(
                bot_name=bot_name,
                bot_help_subtitle=bot_help_subtitle,
                bot_help_image=self.teams.people.me().avatar)
            self.help_command.commands = self.commands
            # Set default help message
            self.help_message = "Hello!  I understand the following commands:  \n"
        else:
            self.help_command = help_command
        self.commands = {
            self.help_command
        }
        if include_demo_commands:
            self.add_command(EchoCommand())

        self.card_callback_commands = {}
        self.approved_users = approved_users
        self.approved_domains = approved_domains
        self.approved_rooms = approved_rooms
        self.approval_parameters_check()
        self.bot_display_name = ""
        self.get_me_info()
        self.threads = threads

    @backoff.on_exception(backoff.expo, requests.exceptions.ConnectionError)
    def get_me_info(self):
        """
        Fetch me info from webexteamssdk
        """
        me = self.teams.people.me()
        self.bot_display_name = me.displayName
        log.info(f"Running as {me.type} '{me.displayName}' with email {me.emails}")
        log.debug(f"Running as bot '{me}'")

    def add_command(self, command_class: Command):
        """
        Add a new command to the bot
        @param command_class: Command Class to add
        """

        for c in self.commands:
            log.debug(f"Checking command '{c}' against {command_class}")
            new_callback_keyword = command_class.card_callback_keyword
            if new_callback_keyword and c.card_callback_keyword == new_callback_keyword:
                raise Exception(f"Error adding new command: '{command_class.command_keyword}'. "
                                f"Duplicate callback_keyword found: "
                                f"'{new_callback_keyword}'. Use a unique keyword in your "
                                f"'{command_class.command_keyword}' adaptive card JSON.")

        self.commands.add(command_class)
        for chained_command in command_class.chained_commands:
            self.commands.add(chained_command)

    def approval_parameters_check(self):
        """
        Simply logs a warning if no approved users, domains or rooms are set.
        """
        if len(self.approved_users) == 0 and len(self.approved_domains) == 0 and len(self.approved_rooms) == 0:
            log.warning("Your bot is open to anyone on Webex Teams. "
                        "Consider limiting this to specific users, domains or room members via the "
                        "WebexBot(approved_domains=['example.com'], approved_users=['user@company.com'], "
                        "approved_rooms=['Y2lzY29zcGFyazovL3VzL1JPT00vZDUwMDE2ZWEtNmQ5My00MTY1LTg0ZWEtOGNmNTNhYjA3YzA5']) "
                        "bot parameters.")

    def check_user_approved(self, user_email, approved_rooms):
        """
        A user is approved if they are in an approved domain or the approved_users list.

        * If both those lists are empty, the user is approved.

        Throws BotException if user is not approved.

        @param approved_rooms: list of spaces the user needs to be in to use this command.
        @param user_email: The email from the user of the incoming message.
        """
        user_approved = False
        self.approval_parameters_check()

        if len(self.approved_users) == 0 and len(self.approved_domains) == 0 and len(approved_rooms) == 0:
            user_approved = True
        elif len(self.approved_domains) > 0 and user_email.split('@')[1] in self.approved_domains:
            user_approved = True
        elif len(self.approved_users) > 0 and user_email in self.approved_users:
            user_approved = True
        elif len(approved_rooms) > 0 and self.is_user_member_of_room(user_email, approved_rooms):
            user_approved = True

        if not user_approved:
            log.warning(f"{user_email} is not approved to interact at this time. Ignoring.")
        return user_approved

    def is_user_member_of_room(self, user_email, approved_rooms):
        is_user_member = False

        for approved_room in approved_rooms:
            try:
                room_members = self.teams.memberships.list(roomId=approved_room, personEmail=user_email)
                for member in room_members:
                    if member.personEmail == user_email:
                        is_user_member = True
            except webexteamssdk.exceptions.ApiError as apie:
                log.warn(f"API error: {apie}")
        return is_user_member

    def process_incoming_card_action(self, attachment_actions, activity):
        """
        Process an incoming card action, determine the command and action,
        and determine reply.
        :param attachment_actions: The attachment_actions object
        :param activity: The websocket activity object
        :return:
        """
        callback_keyword = attachment_actions.inputs.get(CALLBACK_KEYWORD_KEY)
        command_keyword = attachment_actions.inputs.get(COMMAND_KEYWORD_KEY)
        is_card_callback_command = callback_keyword is not None
        raw_message = callback_keyword if callback_keyword else command_keyword
        log.debug(f"raw_message (callback) ='{raw_message}' is_card_callback_command={is_card_callback_command}")

        self.process_raw_command(raw_message,
                                 attachment_actions, activity['actor']['emailAddress'], activity,
                                 is_card_callback_command=is_card_callback_command)

    def process_incoming_message(self, teams_message, activity):
        """
        Process an incoming message, determine the command and action,
        and determine reply.
        :param teams_message: The teams_message object
        :param activity: The websocket activity object
        :return:
        """
        user_email = teams_message.personEmail
        raw_message = teams_message.text
        is_one_on_one_space = 'ONE_ON_ONE' in activity['target']['tags']

        if activity['actor']['type'] != 'PERSON':
            log.debug('message is from a bot, ignoring')
            return

        # Log details on message
        log.info(f"Message from {user_email}: {teams_message}")

        if not self.check_user_approved(user_email=user_email, approved_rooms=self.approved_rooms):
            return

        # Remove the Bots display name from the message if this is not a 1-1
        if not is_one_on_one_space:
            raw_message = raw_message.replace(self.bot_display_name, '').strip()

        self.process_raw_command(raw_message, teams_message, user_email, activity)

    def process_raw_command(self, raw_message, teams_message, user_email, activity, is_card_callback_command=False):
        room_id = teams_message.roomId
        is_one_on_one_space = 'ONE_ON_ONE' in activity['target']['tags']

        # Find the command that was sent, if any
        command = None
        user_command = raw_message.lower()
        log.info(f"New user_command: {user_command}")
        log.info(f"is_card_callback_command: {is_card_callback_command}")

        for c in self.commands:
            log.debug("--------")
            log.debug(f"Checking c.command_keyword: {c.command_keyword}")

            if not is_card_callback_command and c.command_keyword:
                log.debug(f"c.command_keyword: {c.command_keyword}")
                if user_command.find(c.command_keyword) != -1:
                    command = c
                    # If a command was found, stop looking for others
                    break
            else:
                log.debug(f"card_callback_keyword: {c.card_callback_keyword}")
                if user_command == c.command_keyword or user_command == c.card_callback_keyword:
                    command = c
                    break

        if not command:
            log.warning(f"Did not find command for {user_command}. Default to help card.")
            command = self.help_command
        else:
            log.info(f"Found command: {command.command_keyword}")

            if command.approved_rooms:
                if not self.check_user_approved(user_email=user_email, approved_rooms=command.approved_rooms):
                    log.info(f"{user_email} is not allowed to run command: '{command.command_keyword}'")
                    return

        # Build the reply to the user
        reply = ""
        reply_one_to_one = False
        message_without_command = WebexBot.get_message_passed_to_command(command.command_keyword, raw_message)
        thread_parent_id = None

        if hasattr(teams_message, "inputs") and teams_message.inputs.get("thread_parent_id"):
            thread_parent_id = teams_message.inputs.get("thread_parent_id")
        elif 'parent' in activity:
            log.info(f"activity: {activity}")

            if activity['parent']['type'] == 'reply':
                thread_parent_id = activity['parent']['id']
            else:
                # Some bug where message cannot be sent back in response to cardAction in thread.
                # Must reply outside of the thread in this case.
                log.warning(f"There is a server side bug where message cannot be sent back in "
                            f"response to cardAction inside a thread. "
                            f"Must reply outside of the thread in this case.: {activity}")
                thread_parent_id = None
        elif 'id' in activity:
            thread_parent_id = activity['id']
        else:
            log.info("There is no activity id (thread ID) for this request.")

        if command.delete_previous_message and hasattr(teams_message, 'messageId'):
            previous_message_id = teams_message.messageId
            log.info(f"delete_previous_message is True. Deleting message with ID: {previous_message_id}")
            self.teams.messages.delete(previous_message_id)

        if not is_card_callback_command and command.card is not None:
            response = Response()
            response.text = "This bot requires a client which can render cards."
            response.attachments = {
                "contentType": "application/vnd.microsoft.card.adaptive",
                "content": command.card
            }

            pre_card_load_reply, pre_card_load_reply_one_to_one = self.run_pre_card_load_reply(command=command,
                                                                                               message=message_without_command,
                                                                                               teams_message=teams_message,
                                                                                               activity=activity)
            self.do_reply(pre_card_load_reply, room_id, user_email, pre_card_load_reply_one_to_one, is_one_on_one_space, thread_parent_id)
            reply = response
        else:
            log.debug(f"Going to run command: '{command}' with input: '{message_without_command}'")
            pre_execute_reply, pre_execute_reply_one_to_one = self.run_pre_execute(command=command,
                                                                                   message=message_without_command,
                                                                                   teams_message=teams_message,
                                                                                   activity=activity)
            self.do_reply(pre_execute_reply, room_id, user_email, pre_execute_reply_one_to_one, is_one_on_one_space, thread_parent_id)
            reply, reply_one_to_one = self.run_command_and_handle_bot_exceptions(command=command,
                                                                                 message=message_without_command,
                                                                                 teams_message=teams_message,
                                                                                 activity=activity)
        log.info(f"Using thread id={thread_parent_id}")
        return self.do_reply(reply, room_id, user_email, reply_one_to_one, is_one_on_one_space, thread_parent_id)

    def do_reply(self, reply, room_id, user_email, reply_one_to_one, is_one_on_one_space, conv_target_id):
        # allow command handlers to craft their own Teams message
        if reply and isinstance(reply, Response):
            # If the Response lacks a roomId, set it to the incoming room
            if not reply.roomId:
                reply.roomId = room_id
            if not reply.parentId and conv_target_id and self.threads:
                reply.parentId = conv_target_id
            reply = reply.as_dict()
            self.teams.messages.create(**reply)
            reply = "ok"
        # Support returning a list of Responses
        elif reply and isinstance(reply, list):
            for response in reply:
                # Make sure is a Response
                if isinstance(response, Response):
                    if not response.roomId:
                        response.roomId = room_id
                    if not response.parentId and conv_target_id:
                        response.parentId = conv_target_id
                    self.teams.messages.create(**response.as_dict())
                else:
                    # Just a plain message
                    self.send_message_to_room_or_person(user_email,
                                                        room_id,
                                                        reply_one_to_one,
                                                        is_one_on_one_space,
                                                        response,
                                                        conv_target_id)
            reply = "ok"
        elif reply:
            self.send_message_to_room_or_person(user_email,
                                                room_id,
                                                reply_one_to_one,
                                                is_one_on_one_space,
                                                reply,
                                                conv_target_id)
        return reply

    def send_message_to_room_or_person(self,
                                       user_email,
                                       room_id,
                                       reply_one_to_one,
                                       is_one_on_one_space,
                                       reply,
                                       conv_target_id):
        default_move_to_one_to_one_heads_up = \
            quote_info(f"{user_email} I've messaged you 1-1. Please reply to me there.")
        if reply_one_to_one:
            if not is_one_on_one_space:
                if self.threads:
                    self.teams.messages.create(roomId=room_id,
                                               markdown=default_move_to_one_to_one_heads_up,
                                               parentId=conv_target_id)
                else:
                    self.teams.messages.create(roomId=room_id,
                                               markdown=default_move_to_one_to_one_heads_up)
            if self.threads:
                self.teams.messages.create(toPersonEmail=user_email,
                                           markdown=reply,
                                           parentId=conv_target_id)
            else:
                self.teams.messages.create(toPersonEmail=user_email,
                                           markdown=reply)
        else:
            if self.threads:
                self.teams.messages.create(roomId=room_id, markdown=reply, parentId=conv_target_id)
            else:
                self.teams.messages.create(roomId=room_id, markdown=reply)

    def run_pre_card_load_reply(self, command, message, teams_message, activity):
        """
        This allows a reply to be sent back before the command/card function is called. Useful if it takes a while for the card to generate.
        """
        try:
            return command.pre_card_load_reply(message, teams_message, activity), False
        except BotException as e:
            log.warn(f"BotException: {e.debug_message}")
            return e.reply_message, e.reply_one_to_one

    def run_pre_execute(self, command, message, teams_message, activity):
        """
        This allows a reply to be sent back before the execute function is called. Useful if it takes a while to run.
        """
        try:
            return command.pre_execute(message, teams_message, activity), False
        except BotException as e:
            log.warn(f"BotException: {e.debug_message}")
            return e.reply_message, e.reply_one_to_one

    def run_command_and_handle_bot_exceptions(self, command, message, teams_message, activity):
        try:
            return command.card_callback(message, teams_message, activity), False
        except BotException as e:
            log.warn(f"BotException: {e.debug_message}")
            return e.reply_message, e.reply_one_to_one

    @staticmethod
    def get_message_passed_to_command(command, message):
        """
        Remove the command from the start of the message

        :param command: command string
        :param message: message string
        :return: message without command prefix
        """

        if command and message.lower().startswith(command.lower()):
            return message[len(command):]
        return message



import logging

from webexteamssdk.models.cards import Colors, TextBlock, FontWeight, FontSize, Column, AdaptiveCard, ColumnSet, \
    Text, Image, HorizontalAlignment
from webexteamssdk.models.cards.actions import Submit

from webex_bot_ecoen66.formatting import quote_info
from webex_bot_ecoen66.models.command import Command
from webex_bot_ecoen66.models.response import response_from_adaptive_card

log = logging.getLogger(__name__)


class EchoCommand(Command):

    def __init__(self):
        super().__init__(
            command_keyword="echo",
            help_message="Echo Words Back to You!",
            chained_commands=[EchoCallback()])

    def pre_execute(self, message, attachment_actions, activity):
        """
        (optional function).
        Reply before running the execute function.

        Useful to indicate the bot is handling it if it is a long running task.

        :return: a string or Response object (or a list of either). Use Response if you want to return another card.
        """

        image = Image(url="https://i.postimg.cc/2jMv5kqt/AS89975.jpg")
        text1 = TextBlock("Working on it....", weight=FontWeight.BOLDER, wrap=True, size=FontSize.DEFAULT,
                          horizontalAlignment=HorizontalAlignment.CENTER, color=Colors.DARK)
        text2 = TextBlock("I am busy working on your request. Please continue to look busy while I do your work.",
                          wrap=True, color=Colors.DARK)
        card = AdaptiveCard(
            body=[ColumnSet(columns=[Column(items=[image], width=2)]),
                  ColumnSet(columns=[Column(items=[text1, text2])]),
                  ])

        return response_from_adaptive_card(card)

    def execute(self, message, attachment_actions, activity):
        """
        If you want to respond to a submit operation on the card, you
        would write code here!

        You can return text string here or even another card (Response).

        This sample command function simply echos back the sent message.

        :param message: message with command already stripped
        :param attachment_actions: attachment_actions object
        :param activity: activity object

        :return: a string or Response object (or a list of either). Use Response if you want to return another card.
        """

        text1 = TextBlock("Echo", weight=FontWeight.BOLDER, size=FontSize.MEDIUM)
        text2 = TextBlock("Type in something here and it will be echo'd back to you. How useful is that!",
                          wrap=True, isSubtle=True)
        input_text = Text(id="message_typed", placeholder="Type something here", maxLength=30)
        input_column = Column(items=[input_text], width=2)

        submit = Submit(title="Submit",
                        data={
                            "callback_keyword": "echo_callback"})

        card = AdaptiveCard(
            body=[ColumnSet(columns=[Column(items=[text1, text2], width=2)]),
                  ColumnSet(columns=[input_column]),
                  ], actions=[submit])

        return response_from_adaptive_card(card)


class EchoCallback(Command):

    def __init__(self):
        super().__init__(
            card_callback_keyword="echo_callback",
            delete_previous_message=True)

    def execute(self, message, attachment_actions, activity):
        return quote_info(attachment_actions.inputs.get("message_typed"))



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = ["webexteamssdk==1.6.1", "coloredlogs", "websockets==11.0.3", "backoff"]

setup_requirements = ["pytest-runner"]

test_requirements = ["pytest>=3"]

setup(
    author="Finbarr Brady",
    author_email="finbarr.brady@gmail.com",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    description="Python package for a Webex Bot based on websockets.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    include_package_data=True,
    long_description_content_type="text/markdown",
    keywords="webex_bot_ecoen66",
    name="webex_bot_ecoen66",
    packages=find_packages(include=["webex_bot_ecoen66", "webex_bot_ecoen66.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/ecoen66/webex_bot",
    version="0.4.3",
    zip_safe=False,
)



import json

from webexteamssdk.models.cards import AdaptiveCard


def response_from_adaptive_card(adaptive_card: AdaptiveCard):
    """
    Convenience method for generating a Response from an AdaptiveCard.

    @param adaptive_card: AdaptiveCard object
    @return: Response object
    """

    response = Response()
    response.text = "This bot requires a client which can render cards."
    response.markdown = "This bot requires a client which can render cards."
    response.attachments = {
        "contentType": "application/vnd.microsoft.card.adaptive",
        "content": adaptive_card.to_dict()
    }

    return response


class Response(object):
    def __init__(self, attributes=None):
        if attributes:
            self.attributes = attributes
        else:
            self.attributes = dict()
            self.attributes["text"] = None
            self.attributes["roomId"] = None
            self.attributes["parentId"] = None
            self.attributes["markdown"] = None
            self.attributes["html"] = None
            self.attributes["files"] = list()
            self.attributes["attachments"] = list()

    @property
    def text(self):
        return self.attributes["text"]

    @text.setter
    def text(self, val):
        self.attributes["text"] = val

    @property
    def files(self):
        return self.attributes["files"]

    @files.setter
    def files(self, val):
        self.attributes["files"].append(val)

    @property
    def attachments(self):
        return self.attributes["attachments"]

    @attachments.setter
    def attachments(self, val):
        self.attributes["attachments"].append(val)

    @property
    def roomId(self):
        return self.attributes["roomId"]

    @roomId.setter
    def roomId(self, val):
        self.attributes["roomId"] = val

    @property
    def parentId(self):
        return self.attributes["parentId"]

    @parentId.setter
    def parentId(self, val):
        self.attributes["parentId"] = val

    @property
    def markdown(self):
        return self.attributes["markdown"]

    @markdown.setter
    def markdown(self, val):
        self.attributes["markdown"] = val

    @property
    def html(self):
        return self.attributes["html"]

    @html.setter
    def html(self, val):
        self.attributes["html"] = val

    def as_dict(self):
        ret = dict()
        for k, v in self.attributes.items():
            if v:
                ret[k] = v
        return ret

    def json(self):
        return json.dumps(self.attributes)



import logging
from abc import ABC, abstractmethod

log = logging.getLogger(__name__)

CALLBACK_KEYWORD_KEY = 'callback_keyword'
COMMAND_KEYWORD_KEY = "command_keyword"


class Command(ABC):

    def __init__(self, command_keyword=None, chained_commands=[], card=None, help_message=None,
                 delete_previous_message=False,
                 card_callback_keyword=None, approved_rooms=None):
        """
        Create a new bot command.

        If the command has a simple string keyword, it can be invoked by the user when they type that word.

        Alternatively, if `card_callback_keyword` is defined, the command can be invoked from the 'callback_keyword'
         data from the Submit action of a previous card. E.g.

            Submit(title="Submit", data={
                    "callback_keyword": "echo_callback"}
                    )

        @param command_keyword: (optional) Text indicating a phrase to invoke this card.
        @param chained_commands: (optional) List of other commands related
        to this command. This allows multiple related cards to be added at once.
        @param card: (deprecated) A dict representation of the JSON card.
        Prefer to use cards built in code using the webexteamssdk.models.cards classes
        (see https://github.com/fbradyirl/webex_bot/blob/main/webex_bot/commands/echo.py for example)
        @param help_message: Short description of this command.
        @param delete_previous_message: If True, the card which invoked this command will be deleted. (default False)
        @param card_callback_keyword: (optional) this command can be invoked from the 'callback_keyword'
         text in the data from the Submit action of a previous card.
        @param approved_rooms: If defined, only members of these spaces will be allowed to run this command. Default: None (everyone)
        """
        self.command_keyword = command_keyword
        self.help_message = help_message
        self.card = card
        self.pre_card_callback = self.execute
        self.card_callback = self.execute
        self.card_callback_keyword = card_callback_keyword
        self.delete_previous_message = delete_previous_message
        self.approved_rooms = approved_rooms
        self.chained_commands = chained_commands

        # Now, if this card has a Action.Submit action, let's read the callback keyword,
        # or if it doesnt exist, add it.
        # Only work from the first action for now. Maybe in future support multiple actions.
        if card is not None:
            log.warning(f"[{command_keyword}]. Using a card dict is now deprecated. "
                        f"Switch to use adaptive cards built in code "
                        "using the webexteamssdk.models.cards classes (see "
                        "https://github.com/fbradyirl/webex_bot/blob/main/webex_bot/commands/echo.py for example)")
            if 'actions' in card:
                if len(card['actions']) > 0:
                    first_action = card['actions'][0]
                    if 'type' in first_action and first_action['type'] == 'Action.Submit':
                        if 'data' in first_action and len(first_action['data']) > 0:
                            data = first_action['data']
                            if CALLBACK_KEYWORD_KEY in data:
                                self.card_callback_keyword = first_action['data'][CALLBACK_KEYWORD_KEY]
                            elif COMMAND_KEYWORD_KEY in data:
                                self.command_keyword = first_action['data'][COMMAND_KEYWORD_KEY]
                            else:
                                log.warning(
                                    f"card actions data but no entry for '{CALLBACK_KEYWORD_KEY}' for {command_keyword}")
                                self.set_default_card_callback_keyword()
                        else:
                            log.warning(
                                f"no card actions data so no entry for '{CALLBACK_KEYWORD_KEY}' for {command_keyword}")
                            self.set_default_card_callback_keyword()
                else:
                    log.info(f"No actions defined in this card. command_keyword={command_keyword}")

    def set_default_card_callback_keyword(self):
        if self.card_callback_keyword is None:
            if 'data' not in self.card['actions'][0]:
                self.card['actions'][0]['data'] = {}

            self.card_callback_keyword = f"callback___{self.command_keyword}"
            self.card['actions'][0]['data'][CALLBACK_KEYWORD_KEY] = self.card_callback_keyword

            log.info(
                f"Added default action for '{self.command_keyword}' {CALLBACK_KEYWORD_KEY}={self.card_callback_keyword}")

    def pre_card_load_reply(self, message, attachment_actions, activity):
        pass

    def pre_execute(self, message, attachment_actions, activity):
        pass

    @abstractmethod
    def execute(self, message, attachment_actions, activity):
        pass
